/*
 * internal_cell_blobs_to_sinograms.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_blobs_to_sinograms:wrong_argument";

void
cell_blobs_to_sinograms(mxArray * & sinos, const mxArray * const dims_sinos,
    const mxArray * const blobs, const mxArray * const coeffs);

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 3) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array containing the sinograms, the blobs and one with the coefficients");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[2])) {
    mexErrMsgIdAndTxt(errorID,
        "The third argument should be a Cell array");
    return;
  }

  cell_blobs_to_sinograms(plhs[0], prhs[0], prhs[1], prhs[2]);
}

inline void
cell_blobs_to_sinograms(mxArray * & sinos, const mxArray * const dims_sinos,
    const mxArray * const blobs, const mxArray * const coeffs)
{
  const mxClassID class_id = getMxClassOfCell(blobs);
  cell_allocate_from_dims(sinos, dims_sinos, class_id);
  if (!check_blobs_and_sinograms(sinos, blobs, coeffs))
  {
    return;
  }
  switch (class_id)
  {
    case mxDOUBLE_CLASS:
    {
#pragma omp parallel
      {
        cell_initialize_sse<double>(sinos, 0.0);
        do_cell_blobs_to_sinograms_sse<double>(sinos, blobs, coeffs);
      }
      break;
    }
    case mxSINGLE_CLASS:
    {
#pragma omp parallel
      {
        cell_initialize_sse<float>(sinos, 0.0f);
        do_cell_blobs_to_sinograms_sse<float>(sinos, blobs, coeffs);
      }
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}

