/*
 * internal_cell_sinograms_to_blobs.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_single_sinogram_to_blobs:wrong_argument";

void
cell_single_sinogram_to_blobs(mxArray * out_blobs,
    const mxArray * const sino, const mxArray * const coeffs);

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 3) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array containing the blobs, the sinograms and one with the coefficients");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }

  plhs[0] = (mxArray *) prhs[0];

  cell_single_sinogram_to_blobs(plhs[0], prhs[1], prhs[2]);
}

inline void
cell_single_sinogram_to_blobs(mxArray * blobs,
    const mxArray * const sino, const mxArray * const coeffs)
{
  const mxClassID class_id = getMxClassOfCell(blobs);
  if (class_id != mxGetClassID(sino))
  {
    mexErrMsgIdAndTxt(errorID,
        "The sinogram needs to be an array of coherent floating point type, with the arrays in the Blob cell");
    return;
  }
  if (!check_blobs_and_single_sinogram(sino, blobs, coeffs))
  {
    return;
  }
  switch (class_id)
  {
    case mxDOUBLE_CLASS:
    {
#pragma omp parallel
      {
        do_single_sinogram_to_blobs_sse<double>(blobs, sino, coeffs);
      }
      break;
    }
    case mxSINGLE_CLASS:
    {
#pragma omp parallel
      {
        do_single_sinogram_to_blobs_sse<float>(blobs, sino, coeffs);
      }
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}
