/*
 * internal_cell_sinograms_to_blobs.cpp
 *
 *  Created on: Jul 13, 2012
 *      Author: ben
 */

#include "internal_cell_defs.h"

const char * errorID = "C_FUN:cell_sinograms_to_blobs:wrong_argument";

void
cell_sinograms_to_blobs(mxArray * & blobs, const mxArray * const dims_blobs,
    const mxArray * const sinos, const mxArray * const coeffs);

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 3) {
    mexErrMsgIdAndTxt(errorID,
        "Not enough arguments! you need to provide a cell array containing the blobs, the sinograms and one with the coefficients");
    return;
  }
  if (!mxIsCell(prhs[0])) {
    mexErrMsgIdAndTxt(errorID,
        "The first argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[1])) {
    mexErrMsgIdAndTxt(errorID,
        "The second argument should be a Cell array");
    return;
  }
  if (!mxIsCell(prhs[2])) {
    mexErrMsgIdAndTxt(errorID,
        "The third argument should be a Cell array");
    return;
  }

  cell_sinograms_to_blobs(plhs[0], prhs[0], prhs[1], prhs[2]);
}

inline void
cell_sinograms_to_blobs(mxArray * & blobs, const mxArray * const dims_blobs,
    const mxArray * const sinos, const mxArray * const coeffs)
{
  const mxClassID class_id = getMxClassOfCell(sinos);
  cell_allocate_from_dims(blobs, dims_blobs, class_id);
  if (!check_blobs_and_sinograms(sinos, blobs, coeffs))
  {
    return;
  }
  switch (class_id)
  {
    case mxDOUBLE_CLASS:
    {
#pragma omp parallel
      {
        cell_initialize_sse<double>(blobs, 0.0);
        do_cell_sinograms_to_blobs_sse<double>(blobs, sinos, coeffs);
      }
      break;
    }
    case mxSINGLE_CLASS:
    {
#pragma omp parallel
      {
        cell_initialize_sse<float>(blobs, 0.0f);
        do_cell_sinograms_to_blobs_sse<float>(blobs, sinos, coeffs);
      }
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(errorID,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}
