/**
 * Does exactly the same as 'mean2' function from Imaging toolbox.
 *
 * The aim was to get rid of license requirements, and test possible
 * optimization techniques.
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#ifndef __SSE2__
# error "This function requires SSE2 SIMD extensions to be enabled!"
#endif

#include "mex.h"

#include "internal_array_defs.h"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs != 1) {
    mexErrMsgIdAndTxt("C_FUN:gtImgMeanValue:wrong_argument",
        "Not enough arguments! you need to provide a array");
  }
  switch (mxGetClassID(prhs[0])) {
    case mxDOUBLE_CLASS: {
      /* Pointer to incoming matrix: */
      const double * const __restrict array_in = mxGetPr(prhs[0]);

      /* Incoming image dimensions */
      const mwSize numElems = mxGetNumberOfElements(prhs[0]);

      /* Create a matrix for the return argument */
      plhs[0] = mxCreateDoubleScalar(computeAverage(array_in, numElems));

      break;
    }
    case mxSINGLE_CLASS: {
      /* Pointer to incoming matrix: */
      const float * const __restrict array_in = (const float *)mxGetData(prhs[0]);

      /* Incoming image dimensions */
      const mwSize numElems = mxGetNumberOfElements(prhs[0]);

      /* Create a matrix for the return argument */
      plhs[0] = mxCreateNumericMatrix(1, 1, mxSINGLE_CLASS, mxREAL);
      *(float *)mxGetData(plhs[0]) = computeAverage(array_in, numElems);

      break;
    }
    default: {
      mexErrMsgIdAndTxt("C_FUN:gtImgMeanValue:wrong_argument",
          "Unsupported type for array! Please use floating point types");
    }
  }
}

