classdef Gt6DVolumeToSpotProjector < Gt6DVolumeProjector
    properties
        % Weights
        forwardWeights = {};
        backwardWeights = {};
    end

    methods (Access = public)
        function self = Gt6DVolumeToSpotProjector(initialVolumes, detectorSize, varargin)
            self = self@Gt6DVolumeProjector(initialVolumes, detectorSize, varargin{:});

            self.statistics.add_task('fproj_sum', 'FP Spot Sum time');
            self.statistics.add_task('bproj_dupl', 'BP Spot Duplication time');

            self.statistics.add_task('fproj_pre_weight', 'FP Pre-weight time');
            self.statistics.add_task('bproj_pre_weight', 'BP Pre-weight time');

            self.statistics.add_task('fproj_post_weight', 'FP Post-weight time');
            self.statistics.add_task('bproj_post_weight', 'BP Post-weight time');
        end
    end

    methods (Access = protected)
        function sinograms = projectVolume(self, volumes)
            sinograms = self.projectVolume@Gt6DVolumeProjector(volumes);

            self.statistics.tic('fproj_sum');
            sinograms = gtMathsSumCellVolumes(sinograms);
            self.statistics.toc('fproj_sum');
        end

        function volumes = backprojectVolume(self, sinogram)
            self.statistics.tic('bproj_dupl');
            numGeoms = numel(self.astra_geometries);
            sinograms = cell(numGeoms, 1);
            for n = 1:numGeoms
                sinograms{n} = sinogram;
            end
            self.statistics.toc('bproj_dupl');

            volumes = self.backprojectVolume@Gt6DVolumeProjector(sinograms);
        end

        function y = project(self, x)
            self.statistics.tic('fproj_pre_weight');
            y = internal_cell_prod_copy(x, self.backwardWeights);
            self.statistics.toc('fproj_pre_weight');

            y = self.projectVolume(y);

            self.statistics.tic('fproj_post_weight');
            y = y .* self.forwardWeights;
            self.statistics.toc('fproj_post_weight');
        end

        function y = backproject(self, x)
            self.statistics.tic('bproj_pre_weight');
            y = x .* self.forwardWeights;
            self.statistics.toc('bproj_pre_weight');

            y = self.backprojectVolume(y);

            self.statistics.tic('bproj_post_weight');
            y = internal_cell_prod_assign(y, self.backwardWeights);
            self.statistics.toc('bproj_post_weight');
        end
    end
end