classdef GtOrientationResultView < GtBaseGuiElem
    properties
        test_data;
        result;
        post_result;

        distance_viewer = [];
        distance_comp_viewer = [];
        domains_viewer = [];
        profile_viewer = [];
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public object API - Interaction with outside
    methods (Access = public)
        function self = GtOrientationResultView(test_data, result, post_result, varargin)
            self = self@GtBaseGuiElem();

            self.test_data = test_data;
            self.result = result;
            self.post_result = post_result;

            self.conf.f_title = 'Result Browser';

            self.initGtBaseGuiElem(varargin);
        end

        function display_distance(self)
            if (~isempty(self.distance_viewer) && isvalid(self.distance_viewer))
                figure(self.distance_viewer.conf.h_figure)
            else
                self.distance_viewer = GtVolView(self.post_result.distance_deg_alt);

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Make figure...', ...
                        'Callback', @(evt,src)self.make_distance_figure());

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.distance_viewer));

                self.distance_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_viewer.conf.h_context_menu, ...
                        'Label', 'Plot 3D Volume from here...', ...
                        'Callback', @(evt,src)self.make_distance_3dplot());
            end
        end

        function display_distance_comp(self)
            if (~isempty(self.distance_comp_viewer) && isvalid(self.distance_comp_viewer))
                figure(self.distance_comp_viewer.conf.h_figure)
            else
                self.distance_comp_viewer = GtMultiVolView(self.post_result.distance_comp_deg);

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Make figure...', ...
                        'Callback', @(evt,src)self.make_distance_comp_figure());

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.distance_comp_viewer));

                self.distance_comp_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.distance_comp_viewer.conf.h_context_menu, ...
                        'Label', 'Plot 3D Volume from here...', ...
                        'Callback', @(evt,src)self.make_distance_comp_3dplot());
            end
        end

        function display_domains(self)
            if (~isempty(self.domains_viewer) && all(isvalid(self.domains_viewer)))
                for ii = 1:numel(self.domains_viewer)
                    figure(self.domains_viewer(ii).conf.h_figure)
                end
            else
                if (~isempty(self.domains_viewer))
                    % Let's destroy the valid which is left
                    for ii = find(isvalid(self.domains_viewer))'
                        delete(self.domains_viewer(ii))
                    end
                end

                range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];
                picture = {self.post_result.domains_theo, self.post_result.domains_recon};

                self.domains_viewer = GtVolView.compareVolumes(picture, 'clim', range, 'cmap', jet);

                for ii = 1:numel(self.domains_viewer)
                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Make figure...', ...
                            'Callback', @(evt,src)self.make_domains_figure(ii));

                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Analyse Pixel...', ...
                            'Callback', @(evt,src)self.show_pixel_figure(self.domains_viewer(ii)));

                    self.domains_viewer(ii).conf.h_menu_items{end+1} = ...
                        uimenu(self.domains_viewer(ii).conf.h_context_menu, ...
                            'Label', 'Plot 3D Volume from here...', ...
                            'Callback', @(evt,src)self.make_domains_3dplot(ii));
                end
            end
        end

        function display_profile(self)
            if (~isempty(self.profile_viewer) && isvalid(self.profile_viewer))
                figure(self.profile_viewer.conf.h_figure)
            else
                self.profile_viewer = GtVolView(self.result.solution);

%                 self.profile_viewer.conf.h_menu_items{end+1} = ...
%                     uimenu(self.profile_viewer.conf.h_context_menu, ...
%                         'Label', 'Make figure...', ...
%                         'Callback', @(evt,src)self.make_distance_figure());

                self.profile_viewer.conf.h_menu_items{end+1} = ...
                    uimenu(self.profile_viewer.conf.h_context_menu, ...
                        'Label', 'Analyse Pixel...', ...
                        'Callback', @(evt,src)self.show_pixel_figure(self.profile_viewer));
            end
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Internal methods - Addressable from derived classes
    methods (Access = protected)
        function initGui(obj)
            initGui@GtBaseGuiElem(obj);
            % Let's disable zoom, while configuring
            % must disable zoom before setting impixelinfoval (R2008A)
            zoomOp = zoom(obj.conf.h_figure);
            zoomState = get(zoomOp, 'Enable');
            zoom(obj.conf.h_figure, 'off');

            obj.conf.main_boxes = uiextras.VButtonBox('Parent', obj.conf.currentParent);

            obj.conf.h_profile_button = uicontrol('Parent', obj.conf.main_boxes, ...
                                               'Style', 'pushbutton', ...
                                               'String', 'Grain Profile');
            obj.conf.h_distance_button = uicontrol('Parent', obj.conf.main_boxes, ...
                                               'Style', 'pushbutton', ...
                                               'String', 'Distance (deg)');
            obj.conf.h_distance_comp_button = uicontrol('Parent', obj.conf.main_boxes, ...
                                               'Style', 'pushbutton', ...
                                               'String', 'Distance Components (deg)');
            obj.conf.h_domains_button = uicontrol('Parent', obj.conf.main_boxes, ...
                                               'Style', 'pushbutton', ...
                                               'String', 'Domains');

            % Reset back the zoom to the previous state (before the inclusion)
            set(zoomOp, 'Enable', zoomState);
        end

        function addUICallbacks(self)
            self.addUICallback(self.conf.h_profile_button, 'Callback', ...
                              @(src, evt)display_profile(self), false);
            self.addUICallback(self.conf.h_distance_button, 'Callback', ...
                              @(src, evt)display_distance(self), false);
            self.addUICallback(self.conf.h_distance_comp_button, 'Callback', ...
                              @(src, evt)display_distance_comp(self), false);
            self.addUICallback(self.conf.h_domains_button, 'Callback', ...
                              @(src, evt)display_domains(self), false);
        end

        function resetUiComponents(self)
        end

        function doUpdateDisplay(self)
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public interaction API - Interaction with called viewers
    methods (Access = public)
        function make_distance_figure(self)
            point = self.distance_viewer.clicked_point;
            point = self.distance_viewer.getComplete3DPixelInfo(point);
            plane = self.distance_viewer.conf.plane;
            layer = point(self.distance_viewer.conf.ortDim);

            gtCSMakeFigureForPaper(self.post_result.distance_deg_alt, ...
                'colorbar', 'yes', 'border', 'yes', 'filename', '', ...
                'layer', layer, 'plane', plane)
            fprintf([ ...
                'gtCSMakeFigureForPaper(post_result.distance_deg_alt, ' ...
                    '''colorbar'', ''yes'', ''border'', ''yes'', ' ...
                    '''filename'', '''', ''layer'', %d, ''plane'', ''%s'')\n'], ...
                layer, plane)
        end

        function make_distance_comp_figure(self)
            point = self.distance_comp_viewer.clicked_point;
            point = self.distance_comp_viewer.getComplete3DPixelInfo(point);
            plane = self.distance_comp_viewer.conf.plane;
            layer = point(self.distance_comp_viewer.conf.ortDim);
            component = self.distance_comp_viewer.conf.volumeIndex;

            gtCSMakeFigureForPaper(self.post_result.distance_comp_deg(:, :, :, component), ...
                'colorbar', 'yes', 'border', 'yes', 'filename', '', ...
                'layer', layer, 'plane', plane)
            fprintf([ ...
                'gtCSMakeFigureForPaper(post_result.distance_comp_deg(:, :, :, component), ' ...
                    '''colorbar'', ''yes'', ''border'', ''yes'', ' ...
                    '''filename'', '''', ''layer'', %d, ''plane'', ''%s'')\n'], ...
                layer, plane)
        end

        function make_domains_figure(self, indx)
            point = self.domains_viewer(indx).clicked_point;
            point = self.domains_viewer(indx).getComplete3DPixelInfo(point);
            plane = self.domains_viewer(indx).conf.plane;
            layer = point(self.domains_viewer(indx).conf.ortDim);

            range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];

            gtCSMakeFigureForPaper(self.post_result.domains_theo, 'clim', range, 'layer', layer, 'plane', plane)
            gtCSMakeFigureForPaper(self.post_result.domains_recon, 'clim', range, 'layer', layer, 'plane', plane)
            fprintf([ ...
                'gtCSMakeFigureForPaper(post_result.domains_recon, ' ...
                    '''clim'', [%d %d], ' ...
                    '''layer'', %d, ''plane'', ''%s'')\n'], ...
                range, layer, plane)
        end

        function make_distance_3dplot(self)
            point = self.distance_viewer.clicked_point;
            point = self.distance_viewer.getComplete3DPixelInfo(point);
            gtCSPlotVolumeSlices(self.post_result.distance_deg_alt, point, 'colorbar', 'yes');
        end

        function make_distance_comp_3dplot(self)
            point = self.distance_comp_viewer.clicked_point;
            point = self.distance_comp_viewer.getComplete3DPixelInfo(point);
            component = self.distance_comp_viewer.conf.volumeIndex;
            gtCSPlotVolumeSlices(self.post_result.distance_comp_deg(:, :, :, component), point, 'colorbar', 'yes');
        end

        function make_domains_3dplot(self, indx)
            point = self.domains_viewer(indx).clicked_point;
            point = self.domains_viewer(indx).getComplete3DPixelInfo(point);

            range = [min(self.post_result.domains_theo(:)), max(self.post_result.domains_theo(:))];

            gtCSPlotVolumeSlices(self.post_result.domains_theo, point, 'clim', range)
            gtCSPlotVolumeSlices(self.post_result.domains_recon, point, 'clim', range)
        end

        function show_pixel_figure(self, viewer)
            point = viewer.getComplete3DPixelInfo();
            gtCSAnalyseVoxelProperties(self.test_data, self.result, self.post_result, point);
        end
    end
end