function bl = gtFwdSimBuildDifstackBlobs(blobids, indexes, parameters, stackHSize, stackVSize, Xsize, Ysize, bb, info)
% BUILDDIFSTACKSPOTS
%   bl = gtFwdSimBuildDifstackSpots(spotids, indexes, parameters, stackHSize, stackVSize, Xsize, Ysize, bb, info)
%
% blobids    : spotid from grain
% indexes    :
% parameters : parameters.mat
% stackHSize :
% stackVSize :
% Xsize      :
% Ysize      :
% bb         : bounding boxes
% info
%
% bl         :

    num_blobs = numel(blobids);

    if (~exist('indexes', 'var') || isempty(indexes))
        indexes = 1:num_blobs;
    end

    bl = struct( ...
        'intm', cell(num_blobs, 1), ...
        'mask', cell(num_blobs, 1), ...
        'bbuim', cell(num_blobs, 1), ...
        'bbvim', cell(num_blobs, 1), ...
        'bbwim', cell(num_blobs, 1), ...
        'bbsize', cell(num_blobs, 1) );

    for ii = reshape(indexes, 1, [])
        [blob_vol, blob_bb, blob_mask] = gtFwdSimBuildBlob(blobids(ii), parameters);

        % Essentially zero pads the blob to make it fit into ASTRA's
        % diffraction stack
        shifts_blob = gtFwdSimGetStackShifts(stackHSize, stackVSize, blob_bb, false);
        shifts = [shifts_blob.v, shifts_blob.h, 1];

        blob_bb = blob_bb([2 1 3 5 4 6]);

        % We are applying a padding of one slice (perside) on the w
        % diretion
        bl(ii).intm = gtPlaceSubVolume( ...
            zeros(stackVSize, stackHSize, blob_bb(6)+2), ...
            blob_vol, shifts);
        bl(ii).intm = permute(bl(ii).intm, [2 1 3]);

        bl(ii).mask = gtPlaceSubVolume( ...
            zeros(stackVSize, stackHSize, blob_bb(6)+2), ...
            uint8(blob_mask), shifts);
        bl(ii).mask = permute(logical(bl(ii).mask), [2 1 3]);

        bl(ii).bbsize = size(bl(ii).intm);

        bl(ii).bbuim = [(blob_bb(1) - shifts_blob.h), ...
            (blob_bb(1) - shifts_blob.h + bl(ii).bbsize(1) - 1)];
        bl(ii).bbvim = [(blob_bb(2) - shifts_blob.v), ...
            (blob_bb(2) - shifts_blob.v + bl(ii).bbsize(2) - 1)];
        bl(ii).bbwim = [blob_bb(3)-1, (blob_bb(3) + blob_bb(6))];
    end
end

