function [difstack, shifts] = gtFwdSimBuildDifstackSpots(spotids, indexes, parameters, stackHSize, stackVSize, Xsize, Ysize, bb, info)
% BUILDDIFSTACKSPOTS
%   [difstack, shifts] = gtFwdSimBuildDifstackSpots(spotids, indexes, parameters, stackHSize, stackVSize, Xsize, Ysize, bb, info)
%
% spotids    : spotid from grain
% indexes
% parameters : parameters.mat
% stackHSize :
% stackVSize :
% Xsize      :
% Ysize      :
% bb         : bounding boxes
% info
%
% difstack   :
% shifts     : shifts (h,v) for each spot to fit into the montage

    num_spots = numel(spotids);

    difstack = zeros(stackHSize, num_spots, stackVSize);

    if (~exist('indexes', 'var') || isempty(indexes))
        indexes = 1:num_spots;
    end

    if (~exist('parameters', 'var') || isempty(parameters))
        load('parameters.mat');
    end
    if (~exist('info', 'var') || isempty(info))
        difspotfile = fullfile(parameters.acq.dir, '2_difspot', '00000', 'difspot00001.edf');
        if exist(difspotfile, 'file');
            info = edf_info(difspotfile);
        else
            info = [];
        end
    end

    shifts = gtFwdSimGetStackShifts(stackHSize, stackVSize, bb(indexes, :), true);

    for ii = 1:numel(indexes)
        spotid = spotids(indexes(ii));
        bb_id = bb(indexes(ii), :);

        [spot, ~] = gtFwdSimBuildSpot(spotid, parameters, Xsize, Ysize, bb_id, info);
        % Essentially zero pads the spot to make it fit into ASTRA's
        % diffraction stack

        spot_pad = gtPlaceSubImage2(spot,...
            zeros(stackVSize, stackHSize),...
            shifts.h(ii) + 1, shifts.v(ii) + 1, 'summed');

        difstack(:, indexes(ii), :) = spot_pad';
    end
end
