function proj = gtFwdSimBuildProjGeometry(diff_beam_dirs, gr_center, omegas, bb, parameters, stackHSize, stackVSize, selected)
% FUNCTION proj = gtFwdSimBuildProjGeometry(diff_beam_dirs, gr_center, omegas, bb, parameters, stackHSize, stackVSize, selected)
%

    acq = parameters.acq;
    labgeo = parameters.labgeo;
    samgeo = parameters.samgeo;
    recgeo = parameters.recgeo;

    num_projs = size(diff_beam_dirs, 1);

    spot_shifts = gtFwdSimGetStackShifts(stackHSize, stackVSize, bb, true);

    % Bouning boxes in detector coordinates!
    bbpos_det_grain = [ ...
        (bb(:, 1) - spot_shifts.h), (bb(:, 2) - spot_shifts.v), ...
        stackHSize(ones(num_projs, 1)), stackVSize(ones(num_projs, 1)) ...
        ];

    % Bouning boxes in detector coordinates!
    bbpos_det_full = repmat([1 1 acq.xdet acq.ydet], num_projs, []);
    bbpos_det_abs = repmat(acq.bb, num_projs, []);

    %%% Now we actually build the geometries

    % Calculate the projection geometry for the spots:
    %  - Vector used for ROI grain reconstruction in ASTRA (grain shifted
    %    to center of roi volume)
    proj_geom = gtGeoProjForReconstruction(diff_beam_dirs, ...
        omegas, gr_center, bbpos_det_grain, [], ...
        labgeo, samgeo, recgeo, 'ASTRA_grain');

    % Calculate the projection geometry for the full images:
    %  - Vector describing full projection geometry (full images, grain at
    %    nominal position in sample volume)
    proj_geom_full = gtGeoProjForReconstruction( ...
        diff_beam_dirs, omegas, [], bbpos_det_full, ...
        [], labgeo, samgeo, recgeo, 'ASTRA_full');

    % Geometry for extinction spots
    proj_geom_abs = gtGeoProjForReconstruction([], omegas, [], ...
        bbpos_det_abs, [], labgeo, samgeo, recgeo,  'ASTRA_absorption');

    % diffraction geometry
    proj.geom      = proj_geom;
    proj.geom_full = proj_geom_full;
    proj.geom_abs  = proj_geom_abs;

    proj.num_rows  = stackVSize;
    proj.num_cols  = stackHSize;

    proj.centerpix = gtGeoLab2Sam(gr_center, eye(3), labgeo, recgeo, true);

    use_polyhedron = numel(find(selected)) >= 3;
    vol_size = gtFwdSimPredictVolumeSize(gr_center, omegas(selected), ...
        bb(selected, :), parameters, stackHSize, stackVSize, use_polyhedron);

    proj.vol_size_x = vol_size(2);
    proj.vol_size_y = vol_size(1);
    proj.vol_size_z = vol_size(3);

    proj.shift = round((parameters.acq.bb([3 3 4]) - vol_size) / 2 + proj.centerpix) + 1;
end

