function [spot, bb] = gtFwdSimBuildSpot(spotid, parameters, vol_x_size, vol_y_size, bb, info)
% BUILDSPOT  Builds the spot image starting from the difspotID
%   [spot, bb] = gtFwdSimBuildSpot(spotid, [parameters], [volXsize], [volYsize], [bb], [info])
%
% spotid     : difspotID
% parameters : parameters.mat
% volXsize   : volume Xsize for normalization {1}
% volYsize   : volume Ysize for normalization {1}
% bb         : bounding box from DB
% info       : edf image header info

    if (~exist('parameters','var') || isempty(parameters))
        load('parameters.mat');
    end
    if (~exist('vol_x_size','var') || isempty(vol_x_size))
        vol_x_size = 1;
    end
    if (~exist('vol_y_size','var') || isempty(vol_y_size))
        vol_y_size = 1;
    end
    if (~exist('info', 'var') || isempty(info))
        difspotfile = fullfile(parameters.acq.dir, '2_difspot', '00000', 'difspot00001.edf');
        if exist(difspotfile, 'file');
            info = edf_info(difspotfile);
        else
            info = [];
        end
    end

    if (exist('bb', 'var') && ~isempty(bb))
        info.dim_1 = bb(3);
        info.dim_2 = bb(4);
    end
    [spot, bb] = gtGetSummedDifSpot(spotid, parameters, true, info);

    % here we force the intensity to be the same in all spots
    volume = vol_x_size * vol_x_size * vol_y_size;
    spot = spot / sum(spot(:)) * volume;
end
