function verts = gtFwdSimComputeCircumscribingPolyhedron(gr_center, omegas, bb, parameters, display_figure)
% verts = gtFwdSimComputeCircumscribingPolyhedron(gr_center, omegas, bb, parameters, display_figure)
%

    if (~exist('display_figure', 'var'))
        display_figure = false;
    end

    % We think of spot U/V edges like planes that cut the space in a
    % polyhedron when they intersect, having the grain volume on the
    % inside.
    labgeo = parameters.labgeo;
    recgeo = parameters.recgeo;

    % All functions that use the same omegas, need the same rotation tensors:
    rot_comp_w = gtMathsRotationMatrixComp(labgeo.rotdir', 'col');
    rot_omega = gtMathsRotationTensor(omegas, rot_comp_w);

    gc_pos = gr_center(ones(numel(omegas), 1), :);

    % bb is [x_origin, y_origin, x_size, y_size]

    % vectors bu and bv, in positions left, right, bottom, top
    % bu are the plane normals in the 'u' directions, while bv are the
    % plane normals in 'v' directions.
    % Here we compute their positions in the lab coordinates
    bu_l_pos_lab = gtGeoDet2Lab([(bb(:, 1) + bb(:, 3)), (bb(:, 2) + bb(:, 4)/2)], labgeo, 0);
    bu_r_pos_lab = gtGeoDet2Lab([bb(:, 1), (bb(:, 2) + bb(:, 4)/2)], labgeo, 0);
    bv_b_pos_lab = gtGeoDet2Lab([(bb(:, 1) + bb(:, 3)/2), (bb(:, 2) + bb(:, 4))], labgeo, 0);
    bv_t_pos_lab = gtGeoDet2Lab([(bb(:, 1) + bb(:, 3)/2), bb(:, 2)], labgeo, 0);

    bb_cent_lab = gtGeoDet2Lab([(bb(:, 1) + bb(:, 3)/2), (bb(:, 2) + bb(:, 4)/2)], labgeo, 0);

    % bus and bvs positions in the reconstruction coordinates
    bu_l_pos_rec  = gtGeoLab2Sam(bu_l_pos_lab, rot_omega, labgeo, recgeo, 0, 0) - gc_pos;
    bu_r_pos_rec  = gtGeoLab2Sam(bu_r_pos_lab, rot_omega, labgeo, recgeo, 0, 0) - gc_pos;
    bv_b_pos_rec  = gtGeoLab2Sam(bv_b_pos_lab, rot_omega, labgeo, recgeo, 0, 0) - gc_pos;
    bv_t_pos_rec  = gtGeoLab2Sam(bv_t_pos_lab, rot_omega, labgeo, recgeo, 0, 0) - gc_pos;

    bb_cent_rec = gtGeoLab2Sam(bb_cent_lab, rot_omega, labgeo, recgeo, 0, 0) - gc_pos;

    % Let's finally find the normalized plane normals for the boundaries
    pv_t_rec = gtMathsCross(bb_cent_rec, bu_l_pos_rec);
    pv_t_rec = gtMathsNormalizeVectorsList(pv_t_rec);
    pu_l_rec = gtMathsCross(bb_cent_rec, bv_b_pos_rec);
    pu_l_rec = gtMathsNormalizeVectorsList(pu_l_rec);
    pv_b_rec = gtMathsCross(bb_cent_rec, bu_r_pos_rec);
    pv_b_rec = gtMathsNormalizeVectorsList(pv_b_rec);
    pu_r_rec = gtMathsCross(bb_cent_rec, bv_t_pos_rec);
    pu_r_rec = gtMathsNormalizeVectorsList(pu_r_rec);

    % We now compute the norm of the vector from the grain_center to
    % the closest point on the plane normals
    pv_t_rec_norm = sum(pv_t_rec .* bv_t_pos_rec, 2);
    pv_t_rec = pv_t_rec_norm(:, [1 1 1]) .* pv_t_rec;
    pv_b_rec_norm = sum(pv_b_rec .* bv_b_pos_rec, 2);
    pv_b_rec = pv_b_rec_norm(:, [1 1 1]) .* pv_b_rec;
    pu_l_rec_norm = sum(pu_l_rec .* bu_l_pos_rec, 2);
    pu_l_rec = pu_l_rec_norm(:, [1 1 1]) .* pu_l_rec;
    pu_r_rec_norm = sum(pu_r_rec .* bu_r_pos_rec, 2);
    pu_r_rec = pu_r_rec_norm(:, [1 1 1]) .* pu_r_rec;

    verts = gtMathsGetPolyhedronVerticesFromPlaneNormals( ...
        [pv_t_rec; pv_b_rec; pu_l_rec; pu_r_rec] );

    if (display_figure)
        f = figure();
        ax = axes('parent', f);
        hold(ax, 'on')
        scatter3(ax, verts(:, 1), verts(:, 2), verts(:, 3), 30, 'b', 'filled')
        hold(ax, 'off')
    end
end