function [candidateIDs, spotsCands] = gtFwdSimFindCandidates(spotsCommProps, uvw, segmentedSpots)
% FINDCANDIDATES
% [candidateIDs, spotsCands] = gtFwdSimFindCandidates(spotsCommProps, spotsPositions, segmentedSpots)
%
% considers image number, then position, then area
% uvw : spot position
%
% at this stage we want to include spots which may overlap with the spot we are looking for.
% the omega difference of candidate spots must be smaller than thr_max_offset,
% the size of the candidate spot must be at least half the minimum spot size
% the candidate spot must cover the predicted spot position...

    image_ok = find(...
        (segmentedSpots.CentroidImage >= (uvw(3) - spotsCommProps.thr_max_offset) ) ...
        & (segmentedSpots.CentroidImage <= (uvw(3) + spotsCommProps.thr_max_offset) ));

    % centroid distance
    Xdist = abs(segmentedSpots.BoundingBoxXorigin(image_ok) ...
        + 0.5 * segmentedSpots.BoundingBoxXsize(image_ok) - uvw(1));
    Ydist = abs(segmentedSpots.BoundingBoxYorigin(image_ok) ...
        + 0.5 * segmentedSpots.BoundingBoxYsize(image_ok) - uvw(2));

    Xdist_ok = segmentedSpots.BoundingBoxXsize(image_ok) > 2*(Xdist + 0.5 * spotsCommProps.XsizeMin / 2); % was /2 > (
    Ydist_ok = segmentedSpots.BoundingBoxYsize(image_ok) > 2*(Ydist + 0.5 * spotsCommProps.YsizeMin / 2); % was /2 > (
    position_ok = Xdist_ok & Ydist_ok;

    image_pos_ok = image_ok(position_ok);

    % candidates must have a minimum size
    cand_area = segmentedSpots.BoundingBoxXsize(image_pos_ok).*segmentedSpots.BoundingBoxYsize(image_pos_ok);
    % the size of the candidate spot must be at least half the minimum spot size
    area_ok   = cand_area >= 0.5 * spotsCommProps.XsizeMin .* spotsCommProps.YsizeMin;
    image_area_ok  = image_pos_ok(area_ok);

    candidateIDs = image_area_ok;

    spotsCands = sfGetTableIndexes(segmentedSpots, candidateIDs);
    spotsCands.bbox_area = cand_area;
    spotsCands.thr.area_thr  = 0.5 * spotsCommProps.XsizeMin .* spotsCommProps.YsizeMin;
    spotsCands.thr.image_thr = spotsCommProps.thr_max_offset;
    spotsCands.thr.xdist_thr = (0.5 * spotsCommProps.XsizeMin / 2)*2;
    spotsCands.thr.ydist_thr = (0.5 * spotsCommProps.YsizeMin / 2)*2;
    spotsCands.diff.Xdist         = Xdist;
    spotsCands.diff.Ydist         = Ydist;
    spotsCands.diff.image_ok      = image_ok;
    spotsCands.diff.Xdist_ok      = Xdist_ok;
    spotsCands.diff.Ydist_ok      = Ydist_ok;
    spotsCands.diff.position_ok   = position_ok;
    spotsCands.diff.image_pos_ok  = image_pos_ok ;
    spotsCands.diff.area_ok       = area_ok;
    spotsCands.diff.image_area_ok = image_area_ok;
end

function spotsCands = sfGetTableIndexes(segmentedSpots, candidateIDs)
% spotsCands = sfGetTableIndexes(segmentedSpots, candidateIDs)

    if ~isempty(candidateIDs)
        spotsCands = structfun(@(x) x(candidateIDs, :), segmentedSpots, 'UniformOutput', false);
    else
        spotsCands = [];
    end
end % sfGetTableIndexes
