function ondet = gtFwdSimFindSpotsOnDetector(spotsCommProps, u, v, radius, acq, segbb)
% FINDSPOTSONDETECTOR
%   ondet = gtFwdSimFindSpotsOnDetector(parameters,spotsCommProps,u,v,radius,acq,segbb)
%
% spotsCommProps : diffraction spots common properties
% u              : horizontal coordinate of spot on detector
% v              : vertical coordinate of spot on detector
% radius         : radius from center for spot on detector
% acq            : parameters.acq
% segbb          : parameters.seg.bbox

%%% Get the indices of all spots which fully intersect the active region of the detector
ondet = find(all([ ...
    u > spotsCommProps.Xsize/2, u < acq.xdet - spotsCommProps.Xsize/2, ...
    v > spotsCommProps.Ysize/2, v < acq.ydet - spotsCommProps.Ysize/2, ...
    radius < acq.maxradius], 2));

% Exclude spots which overlap with segmentation boundingbox
segOverlap = find(all([...
    (u + spotsCommProps.Xsize/2 >= segbb(1)), ...
    (u - spotsCommProps.Xsize/2 <= (segbb(1) + segbb(3) - 1)), ...
    (v + spotsCommProps.Ysize/2 >= segbb(2)), ...
    (v - spotsCommProps.Ysize/2 <= (segbb(2) + segbb(4) - 1)) ...
    ], 2));

% indices of spots intersection active area of detector
ondet = setdiff(ondet, segOverlap);

end % end of function
