function proj_bls = gtFwdSimPredictProjectedGrainBBox(gr, sel_bb, sel_omegas, ondet, parameters)
    labgeo = parameters.labgeo;
    samgeo = parameters.samgeo;
    recgeo = parameters.recgeo;

    om_step = gtGetOmegaStepDeg(parameters);

    % We first compute the bbox of the grain volume
    verts_rec = gtFwdSimComputeCircumscribingPolyhedron(gr.center, sel_omegas, sel_bb, parameters);

    % Now we project the vertices to the spots and take the convex hull
    dvecs_lab = gr.allblobs.dvec(ondet, :);
    omegas = gr.allblobs.omega(ondet);
%     uv = gr.allblobs.uv(ondet, :);

    % Detector properties
    detscaleu = 1 / labgeo.pixelsizeu;
    detscalev = 1 / labgeo.pixelsizev;
    Qdet = gtFedDetectorProjectionTensor(labgeo.detdiru', labgeo.detdirv', detscaleu, detscalev);
    uvorig = [labgeo.detrefu, labgeo.detrefv]';

    num_omegas = numel(omegas);
    proj_bls = struct( ...
        'mask', cell(1, num_omegas), ...
        'bbuim', cell(1, num_omegas), ...
        'bbvim', cell(1, num_omegas), ...
        'bbwim', cell(1, num_omegas), ...
        'bbsize', cell(1, num_omegas) ...
        );

    % All functions that use the same omegas, need the same rotation tensors:
    % Notice the sign of W !! This is going to be used for a Sam -> Lab
    rot_comp_w = gtMathsRotationMatrixComp(labgeo.rotdir', 'col');
    rot_omega = gtMathsRotationTensor(-omegas, rot_comp_w);

    ones_verts = ones(size(verts_rec, 1), 1);
    gc_sam_pos = gr.center(ones_verts, :);

    for ii = 1:num_omegas
        gc_lab_pos = gtGeoSam2Lab(gc_sam_pos, rot_omega(:, :, ii), labgeo, samgeo, false);
        verts_lab_pos = gtGeoSam2Lab(verts_rec, rot_omega(:, :, ii), labgeo, recgeo, false) + gc_lab_pos;
        dvec_vers = dvecs_lab(ii .* ones_verts, :);

        verts_det_pos = gtFedPredictUVMultiple([], dvec_vers', verts_lab_pos', ...
            labgeo.detrefpos', labgeo.detnorm', Qdet, uvorig)';

%         f = figure();
%         ax = axes('parent', f);
%         hold(ax, 'on')
%         scatter(ax, verts_det_pos(:, 1), verts_det_pos(:, 2));
%         scatter(ax, uv(ii, 1), uv(ii, 2), 30, 'r');
%         hold(ax, 'off')

        verts_idxs = convhull(verts_det_pos);
        verts_det_pos = verts_det_pos(verts_idxs, :);

        bb_2d = [floor(min(verts_det_pos, [], 1))-1, ceil(max(verts_det_pos, [], 1))+1];
        bb_2d = [bb_2d(1:2), (bb_2d(3:4) - bb_2d(1:2) + 1)];

        mask = zeros(bb_2d(3:4));
        verts_mask_pos = round(verts_det_pos) - bb_2d(ones(numel(verts_idxs), 1), 1:2) + 1;
        verts_mask_indx = verts_mask_pos(:, 1) + bb_2d(3) * verts_mask_pos(:, 2);
        mask(verts_mask_indx) = 1;
        mask = bwconvhull(mask);

%         f = figure();
%         ax = axes('parent', f);
%         imshow(mask, 'parent', ax)

        proj_bls(ii).bbuim = [bb_2d(1), (bb_2d(1) + bb_2d(3) - 1)];
        proj_bls(ii).bbvim = [bb_2d(2), (bb_2d(2) + bb_2d(4) - 1)];
        proj_bls(ii).bbwim = round([omegas(ii), omegas(ii)] / om_step);
        proj_bls(ii).bbsize = [bb_2d(3:4), 1];
        proj_bls(ii).mask = mask;

%         pause
    end
end