function [labXYZ, det2lab] = gtGeoDet2Lab(detUV, labgeo, freevec_flag)
% FUNCTION [labXYZ, det2lab] = gtGeoDet2Lab(detUV, labgeo, freevec_flag)
%
% DETECTOR -> LAB coordinate transformation using arbitrary geometry.
%
% Given the Detector coordinates of a set of points, transforms them into
% Lab coordinates.
%
% INPUT  detUV  - detector coordinates in pixels (n,2)
%
%        Geometry parameters in LAB coordinates:
%        labgeo.detorig    - position of detector origin (pixel {0,0});
%                            size (1,3)
%        labgeo.detdiru    - detector U direction (unit row vector)
%        labgeo.detdirv    - detector V direction (unit row vector)
%        labgeo.pixelsizeu - pixel size U (same unit as position values)
%        labgeo.pixelsizev - pixel size V (same unit as position values)
%        labgeo.det2lab (optional) - transformation matrix from Detector to
%                                    Lab coordinates
%
%        freevec_flag      - 1 for free vectors, 0 for bound vectors
%
% OUTPUT labXYZ            - array of the LAB coordinates (n,3)
%        det2lab           - coordinate transformation matrix

% Coordinate transformation matrix
if (~isfield(labgeo, 'det2lab'))
    det2lab = [labgeo.pixelsizeu * labgeo.detdiru; ...
        labgeo.pixelsizev * labgeo.detdirv] ;
else
    det2lab = labgeo.det2lab;
end


% New coordinates
if (freevec_flag)  % free vectors
    labXYZ = detUV * det2lab;
else   % bound vectors
    labXYZ = detUV * det2lab + labgeo.detorig(ones(size(detUV, 1), 1), :);
end
    
end
