function par_labgeo = gtGeoLabDefaultParameters(acq)
% GTGEOLABDEFAULTPARAMETERS
%     par_labgeo = gtGeoLabDefaultParameters(acq)
%     -------------------------------------------
%     Use of acq.dist, acq.pixelsize, acq.xdet, acq.ydet,
%     acq.no_direct_beam, acq.bb, acq.rotation_axis, acq.rotation_direction,
%     acq.detector_definition, [acq.rotationname]
% 

par_labgeo = [];

switch (acq.rotation_name)
    case 'pmo'
        par_labgeo.rotdir = [0 0 -1];
    case 'diffrz'
        par_labgeo.rotdir = [0 0 1];
    case 'unknown'
        par_labgeo.rotdir = [0 0 -1];
        disp('Verify the parameter ''rotdir'' by yourself')
end

if strcmp(acq.rotation_axis, 'horizontal')
    if strcmp(acq.rotation_direction, 'clockwise')
        % It assumes clockwise rotation for images
        par_labgeo.rotdir = -par_labgeo.rotdir;
    end    
end


if (acq.no_direct_beam && strcmp(acq.detector_definition, 'vertical'))
    % Assume vertical setup; detector is above sample pointing downwards (-Z)
    disp(['No direct beam case with a vetical detector. ' ...
        'Any flip was applied during the scan... Please check if it is true'])
    par_labgeo.detrefpos = [0 acq.dist 0];
    par_labgeo.detdiru   = [0 0 -1];
    par_labgeo.detdirv   = [1 0 0];
    if (strcmp(acq.rotation_direction, 'counterclockwise'))
        % is this really true?
        par_labgeo.detrefpos = -1 * par_labgeo.detrefpos;
        par_labgeo.detdiru   = -1 * par_labgeo.detdiru;
    end
    if (isfield(acq, 'bb') && isempty(acq.bb))
        % guess a sample bounding box
        acq.bb = [1 1 4 4];
        disp('Choose a ''fake'' squared sample bounding box in the left-top corner of the image...')
    end
    % Should guess sample envelope
    par_labgeo.samenvtop = [];
    par_labgeo.samenvbot = [];
    par_labgeo.samenvrad = [];
elseif (~acq.no_direct_beam)
    % inline detector HR camera - suppose it on x-axis
    disp('High resolution direct beam scan...')
    
    par_labgeo.detrefpos = [acq.dist 0 0];
    par_labgeo.detdiru   = [0  1  0];
    par_labgeo.detdirv   = [0  0 -1];
    if (strcmp(acq.rotation_direction, 'counterclockwise'))
        % is this really true?
        par_labgeo.detdiru = -1 * par_labgeo.detdiru;
        par_labgeo.detdirv = -1 * par_labgeo.detdirv;
    end
    % Should take sample envelope from sample bbox (parameters.acq.bb) if
    % exists, otherwise in gtPreprocessing
    if (isfield(acq, 'bb') && ~isempty(acq.bb))
        par_labgeo = gtGeoSamEnvFromAcq(par_labgeo, acq);
    else
        par_labgeo.samenvtop = [];
        par_labgeo.samenvbot = [];
        par_labgeo.samenvrad = [];
    end
elseif (acq.no_direct_beam && strcmp(acq.detector_definition, 'inline'))
    % inline detector - suppose it on x-axis
    % taper scan (far field)
    disp('This looks like a taper scan...')
    par_labgeo.detrefpos = [acq.dist 0 0];
    par_labgeo.detdiru   = [0 1 0];
    par_labgeo.detdirv   = [0 0 -1];

    % Should take sample envelope from sample bbox (parameters.acq.bb) if
    % exists, otherwise in gtPreprocessing
    if (isfield(acq,'bb') && isempty(acq.bb))
        % guess a sample bounding box
        acq.bb = [acq.xdet/2-50 acq.ydet/2-50 100 100];
    end
    par_labgeo = gtGeoSamEnvFromAcq(par_labgeo, acq);
    disp('Choose a squared sample bounding box in the center of the image...')
end

par_labgeo.pixelsizeu = acq.pixelsize;
par_labgeo.pixelsizev = acq.pixelsize;
par_labgeo.detsizeu   = acq.xdet; % number of pixels along the u direction
par_labgeo.detsizev   = acq.ydet; % number of pixels along the v direction

% Beam direction, rot. axis position and detector properties
par_labgeo.beamdir = [1 0 0];
par_labgeo.rotpos  = [0 0 0];
par_labgeo.detrefu = (par_labgeo.detsizeu/2) + 0.5;
par_labgeo.detrefv = (par_labgeo.detsizev/2) + 0.5;

% Coordinate axes definition and Lab unit (for records only)
par_labgeo.deflabX = 'Along the beam direction.';
par_labgeo.deflabY = 'Right-handed from Y=cross(Z,X).';
par_labgeo.deflabZ = 'Along rotation axis. Positive away from sample stage.';
par_labgeo.labunit = 'mm';

% Correct for sample bounding box position - 07-01-2014 by LNervo
if ~isempty(acq.bb)
    par_labgeo.detrefpos(3) = ((acq.bb(2) + acq.bb(4)/2) - par_labgeo.detsizev/2) * par_labgeo.pixelsizev;
    par_labgeo.detrefpos(2) = -((acq.bb(1) + acq.bb(3)/2) - par_labgeo.detsizeu/2) * par_labgeo.pixelsizeu;
end

% Detector angular coverage limits (2theta) for entire sample volume
par_labgeo.detanglemin = 0;
par_labgeo.detanglemax = 45;


end % end of function
