function gtCompileFunctions( force, mfiles, funcs_to_compile )

    error_log = {};
    wrong_names = {};
    force = strcmp(force, 'force');

    tot_keys = fieldnames(mfiles);
    if (isempty(funcs_to_compile) ...
            || (numel(funcs_to_compile) == 1 && strcmp(funcs_to_compile{1}, 'all')) )
        funcs_to_compile = tot_keys;
    else
        valid = true(numel(funcs_to_compile), 1);

        for ii = 1:numel(funcs_to_compile)
            if (~ismember(funcs_to_compile{ii}, tot_keys))
                valid(ii) = false;
                wrong_names(1, end+1) = funcs_to_compile(ii);
            end
        end

        funcs_to_compile = funcs_to_compile(valid);
    end
    funcs_to_compile = reshape(funcs_to_compile, [1 numel(funcs_to_compile)]);

    for funcName = funcs_to_compile
        disp(['Checking function: "' funcName{1} '":'])
        func = mfiles.(funcName{1});

        mat_file_path = func.('in_mfile');
        comp_file_path = func.('out_file');
        [upToDate, msg] = gtCheckFunctionUpToDate(mat_file_path, comp_file_path);
        fprintf(msg);
        if (force || ~upToDate)
            disp(['Compiling function: ' funcName{1}]);

            comp_file_dir = fileparts(comp_file_path);
            try
                gt_mcc(funcName{1}, 'out_path', comp_file_dir)
            catch mexc
                error_log(1, end+1) = funcName;
            end
        end
    end

    for err_msg = error_log
        disp(['Error while compiling: ' err_msg{1}]);
    end
    for wr_name = wrong_names
        disp(['Wrong m-file name: ' wr_name{1}]);
    end
end
