'''
Class to facilitate system calls to video converter

Last update by Yoann Guilhem 27 Feb 2013
'''

import os
import subprocess

class videoConversion(object):

    def __init__(self, verbose = False, input = '', output = '', exe = '', \
            defaultArgs = '-y', inputArgs = '', outputArgs = '', \
            inputFPS = 0, outputFPS = 0, outputQuality = None, videoCodec = ''):
        self.verbose = verbose
        self.input = input
        self.output = output
        self.exe = exe
        self.defaultArgs = defaultArgs
        self.inputArgs = inputArgs
        self.outputArgs = outputArgs
        self.inputFPS = inputFPS
        self.outputFPS = outputFPS
        self.outputQuality = outputQuality
        self.videoCodec = videoCodec

    def SetVerbose(self, verbose = True):
        self.verbose = verbose

    def SetInput(self, str):
        self.input = str

    def SetOutput(self, str):
        self.output = str

    def SetDefaultArgs(self, args):
        self.defaultArgs = args

    def SetInputArgs(self, args):
        self.inputArgs = args

    def ResetInputArgs(self, args):
        self.inputArgs = ''

    def SetOutputArgs(self, args):
        self.outputArgs = args

    def ResetOutputArgs(self, args):
        self.outputArgs = ''

    def SetInputFPS(self, fps):
        self.inputFPS = fps

    def ResetInputFPS(self):
        self.inputFPS = 0

    def SetOutputFPS(self, fps):
        self.outputFPS = fps

    def ResetOutputFPS(self):
        self.outputFPS = 0

    def SetOutputQuality(self, quality):
        self.outputQuality = quality

    def ResetOutputQuality(self):
        self.outputQuality = None

    def SetVideoCodec(self, codec):
        self.videoCodec = codec

    def ResetVideoCodec(self):
        self.videoCodec = ''

    def CheckExe(self, verbose = True):
        self.exe = 'avconv'
        self.defaultArgs = '-y -v error'
        try:
            subprocess.call([self.exe, '-version'], \
                    stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        except OSError as e:
            if e.errno == os.errno.ENOENT:
                self.exe = 'ffmpeg'
                self.defaultArgs = '-y -v 2'
                try:
                    subprocess.call([self.exe, '-version'],\
                            stdout = subprocess.PIPE, stderr = subprocess.PIPE)
                except OSError as e:
                    if e.errno == os.errno.ENOENT:
                        raise SystemError('Neither avconv nor ffmpeg are installed!')
                        self.exe = None
                        self.defaultArgs = ''
                    else:
                        raise SystemError('Something went wrong testing ffmpeg!')
            else:
                raise SystemError('Something went wrong testing avconv!')
        if verbose:
            print 'Audio/video converter:', self.exe

    def Run(self, verbose = False):
        if not self.exe:
            self.CheckExe()
        cmd = [self.exe]
        if self.defaultArgs:
            cmd.extend(self.defaultArgs.split())
        if self.inputFPS:
            cmd.extend(['-r', '%d' % self.inputFPS])
        if self.inputArgs:
            cmd.extend(self.inputArgs.split())
        cmd.extend(['-i', self.input])
        if self.outputFPS:
            cmd.extend(['-r', '%d' % self.outputFPS])
        if self.videoCodec:
            cmd.extend(['-vcodec', self.videoCodec])
        if self.outputQuality and self.exe == 'avconv':
            cmd.extend(['-q', '%f' % self.outputQuality])
        if self.outputArgs:
            cmd.extend(self.outputArgs.split())
        cmd.append(self.output)
        if verbose or self.verbose:
            subprocess.call(cmd)
        else:
            subprocess.call(cmd, \
                    stdout = subprocess.PIPE, stderr = subprocess.PIPE)
